import getBoundingClientRect from "../dom-utils/getBoundingClientRect.js";
import getClippingParent from "../dom-utils/getClippingParent.js";
import getDocumentRect from "../dom-utils/getDocumentRect.js";
import getDocumentElement from "../dom-utils/getDocumentElement.js";
import getViewportRect from "../dom-utils/getViewportRect.js";
import computeOffsets from "../utils/computeOffsets.js";
import rectToClientRect from "../utils/rectToClientRect.js";
import { clippingParent, viewport } from "../enums.js";

var getOverflowOffsets = function getOverflowOffsets(popperClientRect, boundaryClientRect) {
  return {
    top: boundaryClientRect.top - popperClientRect.top,
    bottom: popperClientRect.bottom - boundaryClientRect.bottom,
    left: boundaryClientRect.left - popperClientRect.left,
    right: popperClientRect.right - boundaryClientRect.right
  };
};

var getOverflowRect = function getOverflowRect(popper, area) {
  switch (area) {
    case 'clippingParent':
      var documentElement = getDocumentElement(popper);

      var _clippingParent = getClippingParent(popper);

      return _clippingParent === documentElement ? rectToClientRect(getDocumentRect(documentElement)) : getBoundingClientRect(_clippingParent);

    case 'viewport':
      return rectToClientRect(getViewportRect(popper));

    default:
      return getBoundingClientRect(area);
  }
};

export function detectOverflow(_ref) {
  var state = _ref.state,
      options = _ref.options,
      name = _ref.name;
  var _options$clippingArea = options.clippingArea,
      clippingArea = _options$clippingArea === void 0 ? clippingParent : _options$clippingArea,
      _options$visibleArea = options.visibleArea,
      visibleArea = _options$visibleArea === void 0 ? viewport : _options$visibleArea;
  var popperElement = state.elements.popper;
  var referenceElement = state.elements.reference;
  var popperRect = state.measures.popper;
  var clippingClientRect = getOverflowRect(popperElement, clippingArea);
  var visibleClientRect = getOverflowRect(popperElement, visibleArea);
  var referenceClientRect = getBoundingClientRect(referenceElement);
  var popperOffsets = computeOffsets({
    reference: referenceClientRect,
    element: popperRect,
    strategy: 'absolute',
    placement: state.placement,
    scroll: {
      scrollTop: 0,
      scrollLeft: 0
    }
  });
  var popperClientRect = rectToClientRect(Object.assign({}, popperRect, {}, popperOffsets));
  state.modifiersData[name] = {
    clippingArea: getOverflowOffsets(popperClientRect, clippingClientRect),
    visibleArea: getOverflowOffsets(popperClientRect, visibleClientRect)
  };
  return state;
}
export default {
  name: 'detectOverflow',
  enabled: true,
  phase: 'read',
  fn: detectOverflow,
  requires: ['popperOffsets'],
  data: {}
};